@@ -33,13 +33,15 @@ CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
-
 # -*- coding: utf-8 -*-
 import numpy as np
 import os
 import os.path
 import warnings
 import glob
+import operator
+from pathlib import PurePath
+
 
 from refnx.reflect.structure import (
     Scatterer,
@@ -48,7 +50,12 @@ from refnx.reflect.structure import (
     Component,
     sld_profile,
 )
-from refnx.analysis import Parameters, Parameter, possibly_create_parameter
+from refnx.analysis import (
+    Parameters,
+    Parameter,
+    possibly_create_parameter,
+    sequence_to_parameters,
+)
 from refnx._lib import flatten
 from refnx.reflect import _reflect as refcalc
 
@@ -56,12 +63,6 @@ from refnx.reflect import _reflect as refcalc
 contract_by_area = refcalc._contract_by_area
 
 
-# list of material dispersion curves distributed with refellips
-_pth = os.path.dirname(os.path.abspath(__file__))
-_material_files = glob.glob(os.path.join(_pth, "materials/*.csv"))
-materials = [os.path.basename(m)[:-4] for m in _material_files]
-
-
 class ScattererSE(Scatterer):
     """
     Abstract base class for something that will have a refractive index.
@@ -86,9 +87,32 @@ class ScattererSE(Scatterer):
         return self.complex(None)
 
     def complex(self, wavelength):
-        raise NotImplementedError(
-            "The complex method is not implemented for this subclass"
-        )
+        """
+        Calculate a complex RI
+
+        Parameters
+        ----------
+        wavelength : float
+            wavelength of light in nm
+
+        Returns
+        -------
+        RI : complex
+            refractive index and extinction coefficient
+        """
+        if hasattr(self, "epsilon"):
+            wav = self.wavelength
+            if np.any(wavelength):
+                wav = wavelength
+
+            # convert wavelengths to eV
+            energies = nm_eV_conversion(wav)
+            dispersion = self.epsilon(energies)
+            return np.sqrt(dispersion)
+        else:
+            raise NotImplementedError(
+                "epsilon or complex not defined for this object"
+            )
 
     def __call__(self, thick=0, rough=0, vfsolv=0):
         """
@@ -116,206 +140,6 @@ class ScattererSE(Scatterer):
         return SlabSE(thick, self, rough, name=self.name, vfsolv=vfsolv)
 
 
-class RI(ScattererSE):
-    """
-    Object representing a materials wavelength-dependent refractive index.
-
-    An issue is that optical parameters are supplied in units of micrometers
-    ('cause thats what seems to be used in refractive index repos and
-    cauchy models), the wavelength of the incident radiation is supplied in
-    nanometers (thats typical) and the fitting is done in angstroms. Very
-    unpleasant.
-
-    Parameters
-    ----------
-    dispersion : {str, tuple, np.ndarray)
-        If a string then a dispersion curve will be loaded from a file that
-        the string points to. The file is assumed to be of CSV format, with the
-        first column holding the wavelength (in *microns*), with the second
-        column specifying the refractive index. An optional third column can be
-        present that should hold the extinction coefficient.
-
-        If  `dispersion` has length 2 (float, float), then dispersion[0] points
-        to the refractive index of the material and dispersion[1] points to the
-        extinction coefficient. This refractive index is assumed to be
-        wavelength independent.
-
-        If `dispersion` has length 3, then dispersion[0], dispersion[1],
-        dispersion[2] are assumed to hold arrays specifying the wavelength (in
-        *microns*), refractive index, and extinction coefficient.
-    wavelength : float
-        default wavelength for calculation (nm)
-    name : str, optional
-        Name of material.
-
-    Notes
-    -----
-    An RI object can be used to create a Slab
-    """
-
-    def __init__(self, dispersion=None, wavelength=658, name=""):
-        super(RI, self).__init__(name=name, wavelength=wavelength)
-
-        # _wav is only set if a wavelength dependent dispersion curve is loaded
-        # assumed to be in nm
-        self._wav = None
-        self._RI = None
-        self._EC = None
-
-        if dispersion is None:
-            raise RuntimeError("dispersion must be specified")
-
-        if dispersion is not None:
-            if type(dispersion) is str:
-                if not len(name):
-                    # if there is no name get it from the path
-                    name = os.path.basename(dispersion).split(".")[0]
-
-                vals = np.loadtxt(
-                    dispersion, skiprows=1, delimiter=",", encoding="utf8"
-                ).T
-                self._wav = vals[0]
-                self._RI = vals[1]
-                self._EC = np.zeros_like(self._wav)
-                if len(vals) == 3:
-                    self._EC = vals[2]
-                # convert wavelength from um to nm
-                self._wav = self._wav * 1000
-            elif len(dispersion) == 2:
-                self._RI, self._EC = dispersion
-            elif len(dispersion) == 3:
-                # this is if you have an (3, N) array or tuple specifying
-                # wavelength, RI, extinction coef.
-                # wavelength assumed to be in *nm*
-                self._wav, self._RI, self._EC = dispersion
-                self._wav *= 1000
-            else:
-                raise TypeError("format not recognised")
-
-    @property
-    def parameters(self):
-        return Parameters(name=self.name)
-
-    def complex(self, wavelength):
-        """
-        Calculate a complex RI
-
-        Parameters
-        ----------
-        wavelength : float
-            wavelength of light in nm
-
-        Returns
-        -------
-        RI : complex
-            refractive index and extinction coefficient
-        """
-        wav = self.wavelength
-        if np.any(wavelength):
-            wav = wavelength
-
-        if np.any(self._wav):
-            # return a wavelength from a dispersion curve
-            # TODO - raise a warning if the wavelength supplied is outside the
-            # wavelength range covered by the data file.
-            ri_real = np.interp(wav, self._wav, self._RI)
-            ri_imag = np.interp(wav, self._wav, self._EC)
-            return ri_real + 1j * ri_imag
-
-        else:
-            return self._RI + 1j * self._EC
-
-
-def load_material(material):
-    """
-    Loads a dispersion curve from a file distributed with refellips.
-
-    Parameters
-    ----------
-    material: str
-        One of the materials in ``refellips.materials``
-
-    Returns
-    -------
-    ri: refellips.RI
-
-    Notes
-    -----
-    To get a list of the dispersion curves distributed with refellips examine
-    the entries in ``refellips.materials``.
-    """
-    if material in materials:
-        pth = os.path.join(_pth, "materials", f"{material}.csv")
-        return RI(dispersion=pth)
-    raise ValueError(f"{material} is not in the list of dispersion curves")
-
-
-class Cauchy(ScattererSE):
-    """
-    Cauchy model for wavelength-dependent refractive index.
-
-    Optical parameters are supplied in units of micrometers
-    ('cause thats what seems to be used in refractive index repos and
-    cauchy models), the wavelength of the incident radiation is supplied in
-    nanometers (that's typical) and the fitting is done in angstroms.
-
-    The refractive index is calculated as:
-    ``A + (B * 1000**2) / (wav**2) + (C * 1000**4) / (wav**4)``
-
-    where the factors of 1000 convert from microns to nm.
-
-    Parameters
-    ----------
-    A : float or parameter
-        Cauchy parameter A.
-    B : float or parameter
-        Cauchy parameter B in um^2. Default 0.
-    C : float or parameter
-        Cauchy parameter C in um^4. Default 0.
-    wavelength : float
-        default wavelength for calculation (nm)
-    name : str, optional
-        Name of material.
-    """
-
-    def __init__(self, A, B=0, C=0, wavelength=658, name=""):
-        super().__init__(name=name, wavelength=wavelength)
-        self.A = possibly_create_parameter(A, name=f"{name} - cauchy A")
-        self.B = possibly_create_parameter(B, name=f"{name} - cauchy B")
-        self.C = possibly_create_parameter(C, name=f"{name} - cauchy C")
-        self._parameters = Parameters(name=name)
-        self._parameters.extend([self.A, self.B, self.C])
-
-    @property
-    def parameters(self):
-        return self._parameters
-
-    def complex(self, wavelength):
-        """
-        Calculate a complex RI
-
-        Parameters
-        ----------
-        wavelength : float
-            wavelength of light in nm
-
-        Returns
-        -------
-        RI : complex
-            refractive index and extinction coefficient
-        """
-        wav = self.wavelength
-        if np.any(wavelength):
-            wav = wavelength
-
-        real = (
-            self.A.value
-            + (self.B.value * 1000**2) / (wav**2)
-            + (self.C.value * 1000**4) / (wav**4)
-        )
-        return real + 1j * 0.0
-
-
 class ComponentSE(Component):
     """
     A base class for describing the structure of a subset of an interface.
@@ -381,13 +205,13 @@ class ComponentSE(Component):
 
 class SlabSE(ComponentSE):
     """
-    A slab component has uniform SLD over its thickness
-    from refnx.reflect.structure.Slab
+    A slab component has uniform refractive index over its thickness
+
     Parameters
     ----------
     thick : refnx.analysis.Parameter or float
         thickness of slab (Angstrom)
-    ri : :class:`refellips.ScattererSE`, complex, or float
+    ri : :class:`refellips.ScattererSE`
         (complex) RI of film
     rough : refnx.analysis.Parameter or float
         roughness on top of this slab (Angstrom)
@@ -407,10 +231,10 @@ class SlabSE(ComponentSE):
         self.thick = possibly_create_parameter(
             thick, name=f"{name} - thick", units="Å"
         )
-        if isinstance(ri, ScattererSE):
-            self.ri = ri
-        else:
-            self.ri = RI(ri)
+        if not isinstance(ri, ScattererSE):
+            raise ValueError("ri should be a ScattererSE object")
+        self.ri = ri
+
         self.rough = possibly_create_parameter(
             rough, name=f"{name} - rough", units="Å"
         )
@@ -449,7 +273,9 @@ class SlabSE(ComponentSE):
         if self.ri.dispersive:
             ric = self.ri.complex(getattr(structure, "wavelength", None))
         else:
-            ric = complex(self.ri)
+            ric = self.ri
+
+        ric = complex(ric)
 
         return np.array(
             [
@@ -651,14 +477,7 @@ class StructureSE(Structure):
 
     @property
     def solvent(self):
-        if self._solvent is None:
-            if not self.reverse_structure:
-                solv_slab = self[-1].slabs(self)
-            else:
-                solv_slab = self[0].slabs(self)
-            return RI(complex(solv_slab[-1, 1], solv_slab[-1, 2]))
-        else:
-            return self._solvent
+        return self._solvent
 
     @solvent.setter
     def solvent(self, ri):
@@ -668,8 +487,7 @@ class StructureSE(Structure):
             # don't make a new SLD object, use its reference
             self._solvent = ri
         else:
-            solv = RI(ri)
-            self._solvent = solv
+            raise ValueError("ri must be a ScattererSE object or None")
 
     @property
     def depolarisation_factor(self):
@@ -704,6 +522,12 @@ class StructureSE(Structure):
             )
         super().append(item)
 
+    def reflectivity(self):
+        raise NotImplementedError(
+            "Use refellips.ReflectModelSE to calculate ellipsometric"
+            " parameters"
+        )
+
     def slabs(self, **kwds):
         r"""
         The slab representation of this structure.
@@ -1090,3 +914,15 @@ def overall_ri(ri_A, ri_B, vf_B=0.0, ema="linear", depolarisation_factor=1 / 3):
 
     ri_avg = np.sqrt(E_avg)
     return ri_avg
+
+
+def nm_eV_conversion(val):
+    """
+    Convert wavelength from nm to eV
+    -or-
+    Convert wavelength from nm to eV
+
+    It does both.
+    Visible light has a range of energies from 1.77 (red) to 3.26 (blue) eV
+    """
+    return 1239.841984055 / val
